@@ -12,26 +12,29 @@ using namespace Rcpp;
 
 class MinimalProgressBar: public ProgressBar{
   public:
-  MinimalProgressBar()  {
+  MinimalProgressBar(const string str = "Calculating in process")  {
     _finalized = false;
+    _str = str;
   }
   ~MinimalProgressBar() {}
   void display() {}
   void update(float progress) {
     if (_finalized) return;
     REprintf("\r");
-    REprintf(" Calculating in process...(finished %.2f%)", progress * 100);
+    REprintf("%s", _str.c_str());
+    REprintf("...finished %u%%", (int)(progress * 100));
   }
   void end_display() {
     if (_finalized) return;
     REprintf("\r");
-    
-    REprintf(" Calculating in process...(finished 100.00%)");
+    REprintf("%s", _str.c_str());
+    REprintf("...[finished 100%%]");
     REprintf("\n");
     _finalized = true;
   }
   private:
   bool _finalized;
+  string _str;
 };
 
 #endif
